package com.lowes.tcl.api.pem.model;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * Represents an order in the OMS System.
 * <p>
 * created: Jul 24, 2007
 * 
 * @author <a href="mailto:">Rmaruru </a>
 */
public class InstallerInvoice implements Serializable {

	private static final String DELIM = ":";	
	
    private int projectNbr;
    private int purchaseOrdNbr;
    private String invoiceNbr;
    private int locationNbr;
    private Date additionDate;
    private List detail;
    private float invoiceTotalAmount;
    Order order;
    
	/**
	 * Determines if the supplied Object o is equal to this instance.
	 * <p>
	 * Two Order objects are equal if their identifying properties are equal.
	 * The fields purchaseOrdNbr, invoiceNbr and locationNbr 
	 * together identify an Invoice.
	 * 
	 * @param o
	 * @return true if equal otherwise false.
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object object) {
		if (object == this) return true;
		if (object instanceof InstallerInvoice) {
			InstallerInvoice that = (InstallerInvoice)object;
		    return this.getId().equals(that.getId());
		}

		return false;
	}
    
    /**
     * Returns a hash code value for the object.
     * <p>
     * full description
     * 
     * @return a hash code value for this object.
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return ("Invoice" + this.getId()).hashCode();
    }
	
	/**
     * Getter method for Id.
     * <p>
     * Builds an identifiying String by combining the order's identifiers in a 
     * standard way.  The fields purchaseOrdNbr, invoiceNbr and 
     * locationNbr together identify an Invoice.
     * 
     * @return Returns the id.
     */
    public String getId() {
        return 
        	projectNbr + DELIM + 
        	purchaseOrdNbr + DELIM + 
        	locationNbr;
    }
    
    /**
     * Returns a string that "textually represents" this object.
     * <p>
     * Specifically a string is returned that contains all of the instance
     * variable names and values. The value is derived using the instance
     * variable's toString() method.
     * <p>
     * An example of the exact format: <code>[[i=0][animal=cat]]</code>
     * <p>
     * 
     * @return A String representation of this instance.
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return
    		"][projectNbr=" + projectNbr + 
        	"][purchaseOrdNbr=" + purchaseOrdNbr + 
        	"][invoiceNbr=" + invoiceNbr +
        	"][additionDate=" + additionDate +
            "][locationNbr=" + locationNbr +           
            "][invoiceTotalAmount=" + invoiceTotalAmount +           
            "]]";
    }

    /**
     * @return Returns the additionDate.
     */
    public Date getAdditionDate() {
        return additionDate;
    }

    /**
     * @return Returns the locationNbr.
     */
    public int getLocationNbr() {
        return locationNbr;
    }

    /**
     * @param additionDate
     *            The additionDate to set.
     */
    public void setAdditionDate(Date additionDate) {
        this.additionDate = additionDate;
    }

    /**
     * @param locationNbr
     *            The locationNbr to set.
     */
    public void setLocationNbr(int locationNbr) {
        this.locationNbr = locationNbr;
    }

    /**
     * @param purchaseOrdNbr
     *            The purchaseOrdNbr to set.
     */
    public void setPurchaseOrdNbr(int purchaseOrdNbr) {
        this.purchaseOrdNbr = purchaseOrdNbr;
    }

	public String getInvoiceNbr() {
		return invoiceNbr;
	}

	public void setInvoiceNbr(String invoiceNbr) {
		this.invoiceNbr = invoiceNbr;
	}

	public int getPurchaseOrdNbr() {
		return purchaseOrdNbr;
	}

	public List getDetail() {
		return detail;
	}

	public void setDetail(List detail) {
		this.detail = detail;
	}

	public int getProjectNbr() {
		return projectNbr;
	}

	public void setProjectNbr(int projectNbr) {
		this.projectNbr = projectNbr;
	}

	public float getInvoiceTotalAmount() {
		return invoiceTotalAmount;
	}

	public void setInvoiceTotalAmount(float invoiceTotalAmount) {
		this.invoiceTotalAmount = invoiceTotalAmount;
	}

	public Order getOrder() {
		return order;
	}

	public void setOrder(Order order) {
		this.order = order;
	}
	
	/*
	 * Methods for ordering InstallerInvoice objects on nested
	 * properties.
	 */
	
	public String getVbuName()
	{
		return this.order.getInstaller().getVbuName();
	}

	public int getVbuNumber()
	{
		return this.order.getInstaller().getVbuNumber();
	}

	public String getSaleTypeCode()
	{
		return this.order.getSaleTypeCode();
	}
	
}
