package com.lowes.tcl.api.pem.model;

import java.io.Serializable;

import com.lowes.tcl.app.pemw.model.InvoiceDetail;
import com.lowes.tcl.app.pemw.model.Type;

/**
 * Represents an item in the PEM System.
 * <p>
 * full class description
 * <p>
 * Created: Aug 14, 2007
 * 
 * @author <a href="mailto:">Rmaruru </a>
 */
public class Item implements Serializable {
    
    private int itemNbr;
    private int orderQty;
    private float unitPrc;
    private String deliveryTypId;
    private String shortDescription;
    private Type deliveryLoadType;
    private float unitCost;
    
    /**
	 * Returns a string that "textually represents" this object.
	 * <p>
	 * Specifically a string is returned that contains all of the instance
	 * variable names and values.  The value is derived using the instance
	 * variable's toString() method.
	 * <p>
	 * An example of the exact format: <code>[[i=0][animal=cat]]</code>
	 * <p>
	 * @return A String representation of this instance.
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return
			"[" +
			"[itemNbr=" + itemNbr + "]" +
			"[orderQty=" + orderQty + "]" +
			"[unitPrc=" + unitPrc + "]" +
			"[deliveryTypId=" + deliveryTypId + "]" +
			"[shortDescription=" + shortDescription + "]" +
			"[unitCost=" + unitCost + "]" +
			"]";
	}
    
    /**
     * @return Returns the unitPrc.
     */
    public float getUnitPrc() {
        return unitPrc;
    }
    /**
     * @param unitPrc The unitPrc to set.
     */
    public void setUnitPrc(float unitPrc) {
        this.unitPrc = unitPrc;
    }
    /**
     * @return Returns the itemNbr.
     */
    public int getItemNbr() {
        return itemNbr;
    }
    /**
     * @param itemNbr The itemNbr to set.
     */
    public void setItemNbr(int itemNbr) {
        this.itemNbr = itemNbr;
    }
    /**
     * @return Returns the orderQty.
     */
    public int getOrderQty() {
        return orderQty;
    }
    /**
     * @param orderQty The orderQty to set.
     */
    public void setOrderQty(int orderQty) {
        this.orderQty = orderQty;
    }

	public String getDeliveryTypId() {
		return deliveryTypId;
	}

	public void setDeliveryTypId(String deliveryTypId) {
		this.deliveryTypId = deliveryTypId;
	}

	public String getShortDescription() {
		return shortDescription;
	}

	public void setShortDescription(String shortDescription) {
		this.shortDescription = shortDescription;
	}
	
	public float getExtPrice()
	{
		return this.unitPrc * this.orderQty;
	}

	/**
	 * @return Returns the deliveryLoadType.
	 */
	public Type getDeliveryLoadType() {
		return deliveryLoadType;
	}
	/**
	 * @param deliveryLoadType The deliveryLoadType to set.
	 */
	public void setDeliveryLoadType(Type deliveryLoadType) {
		this.deliveryLoadType = deliveryLoadType;
	}

	/**
	 * @return the unitCost
	 */
	public float getUnitCost() {
		return unitCost;
	}

	/**
	 * @param unitCost the unitCost to set
	 */
	public void setUnitCost(float extCost) {
		this.unitCost = extCost;
	}

	public float getExtCost()
	{
		return this.unitCost * this.orderQty;
	}
	
	public InvoiceDetail toInvoiceDetail() {

		InvoiceDetail invoiceDetail = new InvoiceDetail();
		invoiceDetail.setItemNumber(itemNbr);
		invoiceDetail.setItemQuantity(orderQty);
		invoiceDetail.setItemAmount(new Double(getExtPrice()));
		invoiceDetail.setDeliveryLoadType(deliveryLoadType);
		invoiceDetail.setItemDescription(shortDescription);
		invoiceDetail.setItemCost(new Double(getExtCost() + ""));
		return invoiceDetail;
	}

}
