package com.lowes.tcl.api.pem.model;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;

import com.lowes.ss.lknx.util.DateUtil;
import com.lowes.tcl.api.pem.PemApiConstants;

/**
 * Represents an order in the OMS System.
 * <p>
 * created: Jul 24, 2007
 * 
 * @author <a href="mailto:">Rmaruru </a>
 */
public class Order implements Serializable {
	
	private static final String DELIM = ":";	
	
    private int projectNbr;
    private int purchaseOrdNbr;
    private int receivingReportNbr;
    private int locationNbr;
    private Customer customer;
    private String purchaseOrderDesc;
    private Date additionDate;
    private String saleTypeCode;
    private Status status;
    private String deliveryTypeId;
    private Date custNotificationDate;
    private Date estCustPickupDate;
    private Vbu installer;
    private float totalAmount;
    private boolean problemIndc;
    private boolean suspended;

	/**
	 * Determines if the supplied Object o is equal to this instance.
	 * <p>
	 * Two Order objects are equal if their identifying properties are equal.
	 * The fields purchaseOrdNbr, invoiceNbr and locationNbr 
	 * together identify an Invoice.
	 * 
	 * @param o
	 * @return true if equal otherwise false.
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object object) {
		if (object == this) return true;
		if (object instanceof Order) {
			Order that = (Order)object;
		    return this.getId().equals(that.getId());
		}

		return false;
	}
    
	/**
     * Getter method for Id.
     * <p>
     * Builds an identifiying String by combining the order's identifiers in a 
     * standard way.  The fields purchaseOrdNbr, invoiceNbr and 
     * locationNbr together identify an Invoice.
     * 
     * @return Returns the id.
     */
    public String getId() {
        return 
        	projectNbr + DELIM + 
        	purchaseOrdNbr + DELIM + 
        	locationNbr;
    }    
    
    /**
     * Returns a string that "textually represents" this object.
     * <p>
     * Specifically a string is returned that contains all of the instance
     * variable names and values. The value is derived using the instance
     * variable's toString() method.
     * <p>
     * An example of the exact format: <code>[[i=0][animal=cat]]</code>
     * <p>
     * 
     * @return A String representation of this instance.
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return
        	"[[projectNbr=" + projectNbr +
        	"][purchaseOrdNbr=" + purchaseOrdNbr +
        	"][receivingReportNbr=" + receivingReportNbr +
        	"][status=" + status +
        	"][purchaseOrderDesc=" + purchaseOrderDesc +
        	"][customer=" + customer +
        	"][additionDate=" + additionDate +
            "][saleTypeCode=" + saleTypeCode +
            "][locationNbr=" + locationNbr +
            "][deliveryTypeId=" + deliveryTypeId +
            "][custNotificationDate=" + custNotificationDate +
            "][estCustPickupDate=" + estCustPickupDate +
            "][installer=" + installer +
            "][totalAmount=" + totalAmount +
            "][problemIndc=" + problemIndc +
            "][suspended=" + suspended +
            "]]";
    }

    /**
     * @return Returns the additionDate.
     */
    public Date getAdditionDate() {
        return additionDate;
    }

    /**
     * @return Returns the customer.
     */
    public Customer getCustomer() {
        return customer;
    }

    /**
     * @return Returns the locationNbr.
     */
    public int getLocationNbr() {
        return locationNbr;
    }

    /**
     * @return Returns the projectNbr.
     */
    public int getProjectNbr() {
        return projectNbr;
    }

    /**
     * @return Returns the purchaseOrderDesc.
     */
    public String getPurchaseOrderDesc() {
        return purchaseOrderDesc;
    }

    /**
     * @return Returns the purchaseOrdNbr.
     */
    public int getPurchaseOrdNbr() {
        return purchaseOrdNbr;
    }

    /**
     * @return Returns the receivingReportNbr.
     */
    public int getReceivingReportNbr() {
        return receivingReportNbr;
    }

    /**
     * @return Returns the saleTypeCode.
     */
    public String getSaleTypeCode() {
        return saleTypeCode;
    }

    /**
     * @return Returns the status.
     */
    public Status getStatus() {
        return status;
    }

    /**
     * @param additionDate
     *            The additionDate to set.
     */
    public void setAdditionDate(Date additionDate) {
        this.additionDate = additionDate;
    }

    /**
     * @param customer
     *            The customer to set.
     */
    public void setCustomer(Customer customer) {
        this.customer = customer;
    }

    /**
     * @param locationNbr
     *            The locationNbr to set.
     */
    public void setLocationNbr(int locationNbr) {
        this.locationNbr = locationNbr;
    }

    /**
     * @param projectNbr
     *            The projectNbr to set.
     */
    public void setProjectNbr(int projectNbr) {
        this.projectNbr = projectNbr;
    }

    /**
     * @param purchaseOrderDesc
     *            The purchaseOrderDesc to set.
     */
    public void setPurchaseOrderDesc(String purchaseOrderDesc) {
        this.purchaseOrderDesc = purchaseOrderDesc;
    }

    /**
     * @param purchaseOrdNbr
     *            The purchaseOrdNbr to set.
     */
    public void setPurchaseOrdNbr(int purchaseOrdNbr) {
        this.purchaseOrdNbr = purchaseOrdNbr;
    }

    /**
     * @param receivingReportNbr
     *            The receivingReportNbr to set.
     */
    public void setReceivingReportNbr(int receivingReportNbr) {
        this.receivingReportNbr = receivingReportNbr;
    }

    /**
     * @param saleTypeCode
     *            The saleTypeCode to set.
     */
    public void setSaleTypeCode(String saleTypeCode) {
        this.saleTypeCode = saleTypeCode;
    }

    /**
     * @param status
     *            The status to set.
     */
    public void setStatus(Status status) {
        this.status = status;
    }

    /**
     * Returns the additionDate with the date zeroed.
     * Used for bean ordering based on time.
     * 
     * @return Returns the additionDate with the date zeroed.
     */
    public Date getAdditionTime() {
    	Calendar cal = Calendar.getInstance();
    	cal.setTime(additionDate);
    	cal.set(0, 0, 0);
    	
        return cal.getTime();
    }

	public boolean isSuspended() {
		return suspended;
	}

	public void setSuspended(boolean suspended) {
		this.suspended = suspended;
	}

	public String getDeliveryTypeId() {
		return deliveryTypeId;
	}

	public void setDeliveryTypeId(String deliveryTypeId) {
		this.deliveryTypeId = deliveryTypeId;
	}

	public Date getCustNotificationDate() {
		return custNotificationDate;
	}

	public void setCustNotificationDate(Date custNotificationDate) {
		this.custNotificationDate = custNotificationDate;
	}

	public Date getEstCustPickupDate() {
		return estCustPickupDate;
	}

	public void setEstCustPickupDate(Date estCustPickupDate) {
		this.estCustPickupDate = estCustPickupDate;
	}

	public boolean isProblemIndc() {
		return problemIndc;
	}

	public void setProblemIndc(boolean problemIndc) {
		this.problemIndc = problemIndc;
	}
	
	public Vbu getInstaller() {
		return installer;
	}

	public void setInstaller(Vbu installer) {
		this.installer = installer;
	}

	public float getTotalAmount() {
		return totalAmount;
	}

	public void setTotalAmount(float totalAmount) {
		this.totalAmount = totalAmount;
	}

	public int getOrderType()
	{
		String statusSummary = this.getStatus().getOrderStatusSummary();
		
		if("ESTIMATE".equals(statusSummary) || 
		   "OPEN".equals(statusSummary) ||
		   "DECLINED".equals(statusSummary)) {
			return PemApiConstants.ESTIMATE_ORD_TYP;
		} else {
			return PemApiConstants.PRODUCT_ORD_TYP;
		}
	}
	
	public boolean isExpired()
	{
		if(this.getOrderType() == PemApiConstants.ESTIMATE_ORD_TYP) {
			double daysInStatus = Math.floor(DateUtil.duration(this.getStatus().getOrderStatusDate(), new Date(), Calendar.DATE));
			
			if(daysInStatus > 30) {
				return true;
			}
		}
		
		return false;
	}

	public boolean isCalled()
	{
		Calendar cal = Calendar.getInstance();
		cal.setTime(this.getCustNotificationDate());
		
		
		if(this.getOrderType() == PemApiConstants.ESTIMATE_ORD_TYP) {
			//check if time of day is 00:00:00
			if(cal.get(Calendar.HOUR_OF_DAY) == 0 &&
			   cal.get(Calendar.MINUTE) == 0 &&
			   cal.get(Calendar.SECOND) == 0) {
				return true;
			}
		}
		
		return false;
	}
}
