package com.lowes.tcl.api.pem;

import com.lowes.so.lkn.core.LKNException;
import com.lowes.so.lkn.log.LoggingConstants;
import com.lowes.so.lkn.log.LoggingDelegate;

/**
 * Selects a count of projects which where sold in the given period of time.
 * <p>
 * This action counts projects which acquired SOLD status in the given period of time.
 * <p>
 * Created: Fri Oct 03 14:23:45 EDT 2008
 *
 * @author <a href="mailto:vc.julian.nune@lowes.com">Julian F. Nunez</a>
 */
public abstract class PemDBAction implements LoggingConstants, PemApiConstants {

    private static LoggingDelegate loggingDelegate = 
        new LoggingDelegate("PemDBAction");
    
    protected int storeNumber;

    /**
     * Constructor method.  All subclasses should invoke this constructor
    *  from their constructor methods.
     * 
     * @param aDBConnection
     */
    public PemDBAction(int storeNumber) {

    	this.storeNumber = storeNumber;
    }
    
    /**
     * Handles the workflow of one or more logically related SQL calls.
     * <p>
     * This method performs logging of the parameters and SQL, calls
     * validateParameters, creates the SQL statement usually as a
     * PreparedStatement, executes the statement and stores the results
     * available through getResult. Any ResultSet, Statement, and
     * PreparedStatement objects that are instantiated are explicitly closed.
     * Transaction logic such as Commits and Rollbacks are not included and are
     * typically handled by the caller of this method.
     * 
     * @throws LKNException
     *             If the entire workflow did not complete successfully
     *             typically due to database or parameter related issues.
     * @see com.lowes.so.lkn.db.DBAction#execute()
     */
    public abstract void execute() throws LKNException;

    /**
     * Returns the result of the database action as an Object. 
     * <p>
     * It is the caller's responsibility to know how to cast the object to its 
     * appropriate type based on this implementation.
     * 
     * @return A List of CommunicationEventType objects that matched the criteria.
     */
    public abstract Object getResult();

    /**
     * Logs the message at the specified level.
     * 
     * @param level the level to log at
     * @param message the message to log
     */
    protected static void log(int level, String message) {
        loggingDelegate.log(level, message);
    }
    
}